@@ -18,6 +18,7 @@ from webdriver_manager.chrome import ChromeDriverManager
 from webdriver_manager.firefox import GeckoDriverManager
 from webdriver_manager.microsoft import EdgeChromiumDriverManager
 from webdriver_manager.opera import OperaDriverManager
+from webdriver_manager.utils import ChromeType
 # Self 
 from .navigators import go_to_url, enter_frame, go_to_window
 from .finders import find_element, find_elements, find_from_elements, \
@@ -46,6 +47,8 @@ class BrowserServer:
     __OPTIONS_SWITCHER = {
         "chrome": ChrOptions,
         "headless-chrome": ChrOptions,
+        "chromium": ChrOptions,
+        "headless-chromium": ChrOptions,
         "firefox": FfOptions,
         "opera": OpeOptions,
         "edge": EdgOptions
@@ -54,6 +57,8 @@ class BrowserServer:
     __DRIVER_MANAGER = {
             "chrome": ChromeDriverManager,
             "headless-chrome": ChromeDriverManager,
+            "chromium": ChromeDriverManager,
+            "headless-chromium": ChromeDriverManager,
             "firefox": GeckoDriverManager,
             "edge": EdgeChromiumDriverManager,
             "opera": OperaDriverManager
@@ -62,6 +67,8 @@ class BrowserServer:
     __WEB_DRIVERS = {
             "chrome": webdriver.Chrome,
             "headless-chrome": webdriver.Chrome,
+            "chromium": webdriver.Chrome,
+            "headless-chromium": webdriver.Chrome,
             "firefox": webdriver.Firefox,
             "edge": webdriver.Edge,
             "safari": webdriver.Safari,
@@ -72,7 +79,8 @@ class BrowserServer:
         # Definition of private attributes and references
         # All strings allowed for the browser name and version
         self.__authorized_name_version = ("chrome", "firefox", "opera",
-                                          "edge", "safari", "headless-chrome")
+                                          "edge", "safari", "headless-chrome",
+                                          "chromium", "headless-chromium")
         # Browser window's status
         self.__launched = False
 
@@ -161,21 +169,7 @@ class BrowserServer:
         if self.browser_name == "safari":
             self.__web_driver = BrowserServer.__WEB_DRIVERS[self.browser_name](
                 executable_path=self.driver_path)
-        elif self.browser_name == "chrome":
-            # Hack https://stackoverflow.com/questions/64927909/
-            # failed-to-read-descriptor-from-node-connection-a-device-attached-to-the-system
-            if self.__driver_path is None:
-                self.__driver_path = BrowserServer.__DRIVER_MANAGER[self.browser_name]().install() 
-            option = BrowserServer.__OPTIONS_SWITCHER[self.browser_name]()
-            option.add_experimental_option('excludeSwitches', ['enable-logging'])
-            if self.driver_options:
-                for opt in self.driver_options:
-                    option.add_argument(opt)
-            self.__web_driver = BrowserServer.__WEB_DRIVERS[self.browser_name](
-                    executable_path=self.__driver_path,
-                    options=option
-                )
-        elif self.browser_name == "headless-chrome":
+        elif "headless-chrom" in self.browser_name:
             option = BrowserServer.__OPTIONS_SWITCHER[self.browser_name]()
             option.add_experimental_option('excludeSwitches', ['enable-logging'])
             if self.driver_options:
@@ -183,7 +177,29 @@ class BrowserServer:
                     option.add_argument(opt)
             option.headless = True
             if self.__driver_path is None:
-                self.__driver_path = BrowserServer.__DRIVER_MANAGER[self.browser_name]().install()
+                if "ium" in self.browser_name:
+                    self.__driver_path = BrowserServer.__DRIVER_MANAGER[self.browser_name](
+                        chrome_type=ChromeType.CHROMIUM).install()
+                else:
+                    self.__driver_path = BrowserServer.__DRIVER_MANAGER[self.browser_name]().install()
+            self.__web_driver = BrowserServer.__WEB_DRIVERS[self.browser_name](
+                    executable_path=self.__driver_path,
+                    options=option
+                )
+        elif "chrom" in self.browser_name:
+            # Hack https://stackoverflow.com/questions/64927909/
+            # failed-to-read-descriptor-from-node-connection-a-device-attached-to-the-system
+            if self.__driver_path is None:
+                if "ium" in self.browser_name:
+                    self.__driver_path = BrowserServer.__DRIVER_MANAGER[self.browser_name](
+                        chrome_type=ChromeType.CHROMIUM).install()
+                else:
+                    self.__driver_path = BrowserServer.__DRIVER_MANAGER[self.browser_name]().install()
+            option = BrowserServer.__OPTIONS_SWITCHER[self.browser_name]()
+            option.add_experimental_option('excludeSwitches', ['enable-logging'])
+            if self.driver_options:
+                for opt in self.driver_options:
+                    option.add_argument(opt)
             self.__web_driver = BrowserServer.__WEB_DRIVERS[self.browser_name](
                     executable_path=self.__driver_path,
                     options=option
@@ -216,15 +232,15 @@ class BrowserServer:
         self.__launched = False
         return 0
 
-    def full_screenshot(self, filename: str):
+    def __full_screenshot(self, filename: str):
         return fullpage_screenshot(self.webdriver, filename)
 
     def take_a_screenshot(self, save_to: str = None, is_full_screen: bool = True):
         """
         Take a screenshot and save the file to the given folder
-        Each screenshot is followed by a timestamp.
-        :param is_full_screen:
-        :param save_to:
+        Each screenshot is followed by a timestamp. The file pattern is screenshot-<timestamp>.png
+        :param save_to: path location to save the screenshot to
+        :param is_full_screen: defaulted to 'True' capture the full page or the current screenview
         :return: 0 if success
         """
         try:
@@ -236,7 +252,7 @@ class BrowserServer:
                 log.debug("Try to save to '{}'".format(save_to))
                 filename = os.path.join(save_to, "screenshot-{}.png".format(self.__serve_time()))
             if is_full_screen:
-                result = self.full_screenshot(filename)
+                result = self.__full_screenshot(filename)
             else:
                 result = self.webdriver.get_screenshot_as_file(filename)
             if not result:
@@ -257,18 +273,27 @@ class BrowserServer:
     # Start of convenient usage of the automaton tools
     # Navigation
     def go_to(self, url=None):
+        """Navigate to the given url"""
         return go_to_url(driver=self.webdriver, url=url)
 
-    def enter_frame(self, field=None):
-        return enter_frame(driver=self.webdriver, field=field)
+    def enter_frame(self, field=None, web_element: WebElement = None):
+        """Enter the frame found using the find_element method"""
+        return enter_frame(driver=self.webdriver, field=field, web_element=web_element)
 
     def go_to_window(self, handle=None, title=None):
+        """Switch to a window providing either the window handle or window title"""
         return go_to_window(driver=self.webdriver, handle=handle, title=title)
 
     # Finders
-    def find_element(self, field: dict = None, web_element: WebElement = None):
+    def find_element(self,
+                     field: dict = None,
+                     web_element: WebElement = None,
+                     avoid_move_to: bool = False):
         """Find element using the current webdriver or the provided WebElement"""
-        return find_element(driver=self.webdriver, field=field, web_element=web_element)
+        return find_element(driver=self.webdriver,
+                            field=field,
+                            web_element=web_element,
+                            avoid_move_to=avoid_move_to)
 
     def find_elements(self, field: dict = None, web_element: WebElement = None):
         """Find elements using the current webdriver or the provided WebElement"""
@@ -295,27 +320,41 @@ class BrowserServer:
         return find_sub_element_from_element(element, field=field)
 
     # Actions
-    def fill_element(self, field=None, value=None):
-        return fill_element(driver=self.webdriver, field=field, value=value)
+    def fill_element(self, field=None, web_element=None, value=None):
+        """Fill the element with the value. Use the find_element method to find it"""
+        return fill_element(driver=self.webdriver,
+                            field=field,
+                            web_element=web_element,
+                            value=value)
 
-    def fill_elements(self, fields=None, data=None):
-        return fill_elements(driver=self.webdriver, fields=fields, data=data)
+    def fill_elements(self, fields=None, web_element=None, data=None):
+        """Fill data in the respective field. data and fields keys must match.
+        data keys must be in field keys"""
+        return fill_elements(driver=self.webdriver,
+                             fields=fields,
+                             web_element=web_element,
+                             data=data)
 
     # TODO add unit test
     def click_element(self, field=None, web_element=None):
+        """Perform a click element on the element. Use the find_element method to find it"""
         return click_element(driver=self.webdriver,
                              field=field,
                              web_element=web_element)
 
     def select_in_dropdown(self, field=None, visible_text=None, value=None):
+
         return select_in_dropdown(driver=self.webdriver,
                                   field=field,
                                   visible_text=visible_text,
                                   value=value)
 
-    def move_to(self, web_element: WebElement):
+    def move_to(self, web_element: WebElement,
+                caller_message: str = "From BrowserServer instance."):
+        """Bring the element into the screenplay"""
         return move_to(driver=self.webdriver,
-                       element=web_element)
+                       element=web_element,
+                       caller_message=caller_message)
 
     # TODO add unit test
     def select_in_angular_dropdown(self, root_field=None, visible_text=None):
@@ -338,6 +377,9 @@ class BrowserServer:
                                   field=field,
                                   displayed_text=displayed_text)
 
+    def execute_script(self, script: str, web_element: WebElement):
+        self.webdriver.execute_script(script, web_element)
+
     # Alerts
     def alert_message(self):
         return alert_message(driver=self.webdriver)
@@ -351,32 +393,43 @@ class BrowserServer:
     def is_alert_present(self, until=5):
         return is_alert_present(driver=self.webdriver, until=until)
 
-    def is_field_exist(self, field=None, until=5):
-        return is_field_exist(driver=self.webdriver, field=field, until=until)
+    def is_field_exist(self, field=None, web_element=None, until=5):
+        return is_field_exist(driver=self.webdriver,
+                              field=field,
+                              web_element=web_element,
+                              until=until)
 
-    def is_field_contains_text(self, field=None, text=None):
-        return is_field_contains_text(driver=self.webdriver, field=field, text=text)
+    def is_field_contains_text(self, field=None, web_element=None, text=None):
+        return is_field_contains_text(driver=self.webdriver,
+                                      field=field,
+                                      web_element=web_element,
+                                      text=text)
 
-    def element_text(self, field=None):
-        return element_text(driver=self.webdriver, field=field)
+    def element_text(self, field=None, web_element=None):
+        return element_text(driver=self.webdriver,
+                            field=field,
+                            web_element=web_element)
 
-    def is_field_displayed(self, field=None):
-        return is_field_displayed(driver=self.webdriver, field=field)
+    def is_field_displayed(self, field=None, web_element=None):
+        return is_field_displayed(driver=self.webdriver, field=field, web_element=web_element)
 
-    def is_field_enabled(self, field=None, attribute=None):
-        return is_field_enabled(driver=self.webdriver, field=field, attribute=attribute)
+    def is_field_enabled(self, field=None, web_element=None, attribute=None):
+        return is_field_enabled(driver=self.webdriver, field=field, web_element=web_element,
+                                attribute=attribute)
 
     def how_many_windows(self):
         return how_many_windows(driver=self.webdriver)
 
-    def retrieve_tabular(self, field=None, row_and_col=("tr", "td", "th")):
-        return retrieve_tabular(driver=self.webdriver, field=field, row_and_col=row_and_col)
+    def retrieve_tabular(self, field=None, web_element=None, row_and_col=("tr", "td", "th")):
+        return retrieve_tabular(driver=self.webdriver, field=field, web_element=web_element,
+                                row_and_col=row_and_col)
 
     # TODO add unit test
     def where_am_i(self):
         return where_am_i(driver=self.webdriver)
 
     # TODO add unit test
-    def is_checkbox_checked(self, field=None, is_angular=False):
-        return is_checkbox_checked(driver=self.webdriver, field=field, is_angular=is_angular)
+    def is_checkbox_checked(self, field=None, web_element=None, is_angular=False):
+        return is_checkbox_checked(driver=self.webdriver, field=field, web_element=web_element,
+                                   is_angular=is_angular)
 
