@@ -1,9 +1,10 @@
 # -*- coding: utf-8 -*-
 import logging
+from typing import List, Union
 
 from deprecated.classic import deprecated
 
-from .drivers_tools import driver_field_validation, move_to, web_drivers_tuple, web_element_validation
+from .drivers_tools import driver_field_validation, move_to, web_element_validation
 from selenium.common.exceptions import NoSuchElementException
 from selenium.webdriver.remote.webelement import WebElement
 from selenium.webdriver.common.action_chains import ActionChains
@@ -39,7 +40,10 @@ def __find_elements(web_element, field: dict):
     return switcher[field["type"]](field["value"])
 
 
-def find_element(driver=None, field=None, web_element=None):
+def find_element(driver=None,
+                 field: dict = None,
+                 web_element: WebElement = None,
+                 avoid_move_to: bool = False) -> Union[WebElement, None]:
     """
     Look up for the field described as a dictionary {"type": string, "value":}.
     Example: {"type": "id", "value": "frmCentreNumber"}
@@ -47,6 +51,7 @@ def find_element(driver=None, field=None, web_element=None):
     :param driver: a selenium web driver
     :param field: a dictionary
     :param web_element: a web_element to search from
+    :param avoid_move_to: avoid to move to the found element
     :raise AssertionError: if driver is not a proper web driver instance or
             the field is not a dictionary
     :raise KeyError: If the field variable doesn't contain the expected keys i.e. type and value
@@ -54,35 +59,30 @@ def find_element(driver=None, field=None, web_element=None):
     :return: a selenium web element
     """
     try:
-        driver_field_validation(driver, field)
-        web_element_validation(web_element)
+        driver_field_validation(driver, field, log)
+        web_element_validation(web_element, log)
 
-        if "text" in field.keys():
-            return find_from_elements(driver=driver, field=field, text=field['text'])
+        if "text" in field:
+            return find_from_elements(driver=driver,
+                                      field=field,
+                                      text=field['text'],
+                                      web_element=web_element)
 
         if web_element is None:
             element = __find_element(driver, field)
         else:
             element = __find_element(web_element, field)
 
-        move_to(driver, element)
+        if not avoid_move_to:
+            move_to(driver, element, f"Find element: {field}")
         return element
-    except AttributeError as attribute_error:
-        log.error("finders.find_element raised an assertion with following input"
-                  f" driver:'{driver}' and field:'{field}' and web_element: '{web_element}'."
-                  f" Assertion is '{attribute_error.args}'")
-        raise AttributeError(attribute_error)
-    except KeyError as key_error:
-        log.error(key_error.args[0])
-        raise KeyError(key_error)
     except NoSuchElementException as no_such_element:
-        log.error("In find_element didn't find the element '{}'."
-                  " Exception is '{}'".format(field, no_such_element.args))
-        raise NoSuchElementException("Element designed by field '{}'"
-                                     " could not be located.".format(field)) from None
+        log.warning("In find_element didn't find the element '{}'."
+                    " Exception is '{}'".format(field, no_such_element.args))
+        return None
 
 
-def find_elements(driver=None, field=None, web_element=None):
+def find_elements(driver=None, field=None, web_element=None) -> List[WebElement]:
     """
     Look up for the field described as a dictionary {"type": string, "value":}.
     Example: {"type": "id", "value": "frmCentreNumber"}
@@ -94,33 +94,30 @@ def find_elements(driver=None, field=None, web_element=None):
     :raise KeyError: If the field variable doesn't contain the expected keys i.e. type and value
     :return: a list of selenium web element
     """
-    try:
-        driver_field_validation(driver, field)
-        web_element_validation(web_element)
-        # Define the association between a type and a selenium find action
-        if web_element is not None:
-            return __find_elements(web_element, field)
-        else:
-            return __find_elements(driver, field)
-    except AttributeError as attribute_error:
-        log.error("finders.find_elements raised an assertion with following input"
-                  f" driver:'{driver}' and field:'{field}' and web_element: '{web_element}'."
-                  f" Assertion is '{attribute_error.args}'")
-        raise AttributeError(attribute_error)
-    except KeyError as key_error:
-        log.error(key_error.args[0])
-        raise KeyError(key_error)
+
+    driver_field_validation(driver, field, log)
+    web_element_validation(web_element, log)
+    # Define the association between a type and a selenium find action
+    if web_element is not None:
+        return __find_elements(web_element, field)
+    else:
+        return __find_elements(driver, field)
 
 
-def find_from_elements(driver=None, field=None, text=None, web_element=None):
+def find_from_elements(driver=None,
+                       field=None,
+                       text=None,
+                       web_element=None,
+                       avoid_move_to: bool = False) -> Union[WebElement, None]:
     """
     Try to locate an element using his text
-    :param web_element:
     :param driver: a selenium web driver
     :param field: a dictionary
     :param text: a string
-    :raise AssertionError: from the eaifinders.find_elements method
-    :raise KeyError: from the eaifinders.find_elements method
+    :param web_element: a WebElement to search from
+    :param avoid_move_to: avoid to move to the found element
+    :raise AssertionError: from the finders.find_elements method
+    :raise KeyError: from the finders.find_elements method
     :raise NoSuchElementException: when no element is found
     :return: a selenium web element
     """
@@ -132,7 +129,7 @@ def find_from_elements(driver=None, field=None, text=None, web_element=None):
     if not isinstance(text, str):
         raise AttributeError("text must be a string")
     if not text:
-        raise AttributeError("text must be non-empty")
+        raise ValueError("text must be non-empty")
 
     for element in elements:
         if element.text == text:
@@ -144,14 +141,16 @@ def find_from_elements(driver=None, field=None, text=None, web_element=None):
             return_element = element
             break
 
-    if return_element is None:
-        raise NoSuchElementException(f"Element designed by field '{field}' and text '{text}'"
-                                     " could not be located.")
-    move_to(driver, return_element)
+    if return_element is not None:
+        if not avoid_move_to:
+            move_to(driver, return_element, f"Find from elements '{field}' and '{text}'")
+    else:
+        log.warning(f"Element designed by field '{field}' and text '{text}'"
+                    " could not be located.")
     return return_element
 
 
-@deprecated(version="1.0.5", reason="You should user find_element with a web_element")
+@deprecated(version="1.0.5", reason="You should use find_element with a web_element")
 def find_sub_element_from_element(web_element=None, field=None):
     """
     Return a sub element from the element
@@ -174,7 +173,7 @@ def find_sub_element_from_element(web_element=None, field=None):
             "partial_link_text": web_element.find_element_by_partial_link_text,
             "tag_name": web_element.find_element_by_tag_name,
             "xpath": web_element.find_element_by_xpath
-            }
+        }
 
         element = switcher[field["type"]](field["value"])
         actions = ActionChains(web_element.parent)
